import time

import pytest


def test_destinationsideofpier(device, disconnect):
    assert device.DestinationSideOfPier(12, 45) is not None


def test_findhome(device, disconnect):
    if device.CanFindHome:
        if device.CanUnpark:
            device.Unpark()
        device.FindHome()
        while device.Slewing:
            time.sleep(0.1)
        assert device.AtHome


def test_park(device, disconnect):
    if device.CanPark:
        device.Park()
        while device.Slewing:
            time.sleep(0.1)
        assert device.AtPark


def test_setpark(device, disconnect):
    if device.CanSetPark:
        device.SetPark()


@pytest.mark.skip(
    reason="Synchronous methods are deprecated, not available via Alpaca."
)
def test_slewtoaltaz(device, disconnect):
    if device.CanSlewAltAz:
        device.SlewToAltAz(45, 180)


def test_slewtoaltazasync(device, disconnect):
    if device.CanSlewAltAzAsync:
        if device.CanUnpark:
            device.Unpark()
        if device.CanSetTracking:
            device.Tracking = False
        device.SlewToAltAzAsync(45, 90)
        device.AbortSlew()


@pytest.mark.skip(
    reason="Synchronous methods are deprecated, not available via Alpaca."
)
def test_slewtocoordinates(device, disconnect):
    if device.CanSlew:
        if device.CanUnpark:
            device.Unpark()
        device.SlewToCoordinates(device.SiderealTime, device.SiteLatitude)


def test_slew_tocoordinatesasync(device, disconnect):
    if device.CanSlewAsync:
        if device.CanUnpark:
            device.Unpark()
        device.SlewToCoordinatesAsync(device.SiderealTime, device.SiteLatitude)
        device.AbortSlew()


@pytest.mark.skip(
    reason="Synchronous methods are deprecated, not available via Alpaca."
)
def test_slewtotarget(device, disconnect):
    device.TargetDeclination = device.SiteLatitude
    assert device.TargetDeclination == device.SiteLatitude

    t = device.SiderealTime
    device.TargetRightAscension = t
    assert device.TargetRightAscension == t

    if device.CanUnpark:
        device.Unpark()

    device.SlewToTarget()


def test_slewtotargetasync(device, disconnect):
    device.TargetDeclination = device.SiteLatitude
    assert device.TargetDeclination == device.SiteLatitude

    t = device.SiderealTime
    device.TargetRightAscension = t
    assert device.TargetRightAscension == t

    if device.CanUnpark:
        device.Unpark()

    device.SlewToTargetAsync()
    device.AbortSlew()


def test_synctoaltaz(device, disconnect):
    if device.CanSyncAltAz:
        if device.CanSetTracking:
            device.Tracking = False
        device.SyncToAltAz(45, 90)


def test_synctocoordinates(device, disconnect):
    if device.CanSync:
        if device.CanSetTracking:
            device.Tracking = True
        device.SyncToCoordinates(device.SiderealTime, device.SiteLatitude)


def test_synctotarget(device, disconnect):
    if device.CanSync:
        device.TargetDeclination = device.SiteLatitude
        assert device.TargetDeclination == device.SiteLatitude

        t = device.SiderealTime
        device.TargetRightAscension = t
        assert device.TargetRightAscension == t

        if device.CanSetTracking:
            device.Tracking = True

        device.SyncToTarget()


def test_unpark(device, disconnect):
    if device.CanUnpark:
        device.Unpark()
        assert not device.AtPark


def test_alignmentmode(device, disconnect):
    assert device.AlignmentMode is not None


def test_altitude(device, disconnect):
    assert device.Altitude is not None


def test_aperturearea(device, disconnect):
    assert device.ApertureArea is not None


def test_aperturediameter(device, disconnect):
    assert device.ApertureDiameter is not None


def test_azimuth(device, disconnect):
    assert device.Azimuth is not None


def test_declination(device, disconnect):
    assert device.Declination is not None


def test_declinationrate(device, disconnect):
    if device.CanSetDeclinationRate:
        device.DeclinationRate = 1
        assert device.DeclinationRate == 1


def test_doesrefraction(device, disconnect):
    assert device.DoesRefraction is not None
    device.DoesRefraction = True
    assert device.DoesRefraction


def test_equatorialsystem(device, disconnect):
    assert device.EquatorialSystem is not None


def test_focallength(device, disconnect):
    assert device.FocalLength is not None


def test_guideratedeclination(device, disconnect):
    assert device.GuideRateDeclination is not None
    device.GuideRateDeclination = 1
    assert device.GuideRateDeclination == 1


def test_guideraterightascension(device, disconnect):
    assert device.GuideRateRightAscension is not None
    device.GuideRateRightAscension = 1
    assert device.GuideRateRightAscension == 1


def test_ispulseguiding(device, disconnect):
    assert device.IsPulseGuiding is not None


def test_rightascension(device, disconnect):
    assert device.RightAscension is not None


def test_rightascensionrate(device, disconnect):
    if device.CanSetRightAscensionRate:
        device.RightAscensionRate = 1
        assert device.RightAscensionRate == 1


def test_sideofpier(device, disconnect):
    if device.CanSetPierSide:
        assert device.SideOfPier is not None


def test_siteeleveation(device, disconnect):
    assert device.SiteElevation is not None
    device.SiteElevation = 100
    assert device.SiteElevation == 100


def test_sitelongitude(device, disconnect):
    assert device.SiteLongitude is not None
    device.SiteLongitude = 0
    assert device.SiteLongitude == 0


def test_slewsettletime(device, disconnect):
    assert device.SlewSettleTime is not None
    device.SlewSettleTime = 1
    assert device.SlewSettleTime == 1


def test_tracking(device, disconnect):
    if device.CanSetTracking:
        device.Tracking = True
        assert device.Tracking


@pytest.mark.skip(reason="Alpaca implementation issue")
def test_trackingrate(device, disconnect):
    if device.CanSetTracking:
        assert device.TrackingRate is not None
        device.TrackingRate = device.TrackingRates[0]
        assert device.TrackingRate == device.TrackingRates[0]


def test_utcdate(device, disconnect):
    assert device.UTCDate is not None
    device.UTCDate = "2020-01-01T00:00:00"
    assert device.UTCDate.strftime("%Y-%m-%dT%H:%M:%S") == "2020-01-01T00:00:00"



import logging
import time

import astroscrappy
import click
import numpy as np
from astropy.io import fits

logger = logging.getLogger(__name__)


@click.command(
    epilog="""Check out the documentation at
                https://pyscope.readthedocs.io/ for more
                information."""
)
@click.option(
    "-t",
    "--camera-type",
    type=click.Choice(["ccd", "cmos"]),
    default="ccd",
    show_choices=True,
    show_default=True,
    help="Camera type.",
)
@click.option(
    "-d",
    "--dark-frame",
    type=click.Path(exists=True, resolve_path=True),
    help="Path to master dark frame.",
)
@click.option(
    "-f",
    "--flat-frame",
    type=click.Path(exists=True, resolve_path=True),
    help="Path to master flat frame.",
)
@click.option(
    "-b",
    "--bias-frame",
    type=click.Path(exists=True, resolve_path=True),
    help="Path to master bias frame. Ignored if camera type is CMOS.",
)
@click.option(
    "-a",
    "--flat-dark-frame",
    type=click.Path(exists=True, resolve_path=True),
    help="Path to master flat-dark frame. Ignored if camera type is CCD.",
)
@click.option(
    "-s",
    "--astro-scrappy",
    nargs=2,
    default=(1, 3),
    show_default=True,
    help="Number of hot pixel removal iterations and estimated camera read noise.",
)
@click.option(
    "-c",
    "--bad-columns",
    default="",
    show_default=True,
    help="Comma-separated list of bad columns to fix.",
)
@click.option(
    "-i",
    "--in-place",
    is_flag=True,
    default=False,
    show_default=True,
    help="Overwrite input files.",
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    show_default=True,
    help="Print verbose output.",
)
@click.argument("fnames", nargs=-1, type=click.Path(exists=True, resolve_path=True))
@click.version_option()
def ccd_calib_cli(
    camera_type,
    dark_frame,
    flat_frame,
    bias_frame,
    flat_dark_frame,
    astro_scrappy,
    bad_columns,
    in_place,
    verbose,
    fnames,
):
    if verbose:
        logger.setLevel(logging.DEBUG)

    logger.debug(
        f"""ccd_calib(camera_type={camera_type}, dark_frame={dark_frame},
    flat_frame={flat_frame}, bias_frame={bias_frame}, astro_scrappy={astro_scrappy},
    bad_columns={bad_columns}, in_place={in_place}, fnames={fnames})"""
    )

    if camera_type == "ccd":
        if bias_frame is not None:
            logger.info(f"Loading bias frame: {bias_frame}")
            with fits.open(bias_frame) as hdul:
                bias = hdul[0].data
                hdr = hdul[0].header
            bias = bias.astype(np.float32)

            try:
                bias_readout_mode = hdr["READOUTM"]
            except KeyError:
                bias_readout_mode = hdr["READOUT"]

            try:
                bias_exptime = round(hdr["EXPTIME"], 3)
            except KeyError:
                bias_exptime = round(hdr["EXPOSURE"], 3)

            try:
                bias_xbin = hdr["XBINNING"]
            except:
                bias_xbin = hdr["XBIN"]

            try:
                bias_ybin = hdr["YBINNING"]
            except:
                bias_ybin = hdr["YBIN"]

            logger.debug(f"Bias frame readout mode: {bias_readout_mode}")
            logger.debug(f"Bias frame exposure time: {bias_exptime}")
            logger.debug(f"Bias frame X binning: {bias_xbin}")
            logger.debug(f"Bias frame Y binning: {bias_ybin}")
            logger.debug(f"Bias frame pedestal: {bias_pedestal}")
    elif camera_type == "cmos":
        if flat_dark_frame is not None:
            logger.info(f"Loading flat-dark frame: {flat_dark_frame}")
            with fits.open(flat_dark_frame) as hdul:
                flat_dark = hdul[0].data
                hdr = hdul[0].header
            flat_dark = flat_dark.astype(np.float32)

            try:
                flat_dark_filter = hdr["FILTER"]
            except KeyError:
                flat_dark_filter = ""

            try:
                flat_dark_readout_mode = hdr["READOUTM"]
            except KeyError:
                flat_dark_readout_mode = hdr["READOUT"]

            try:
                flat_dark_exptime = hdr["EXPTIME"]
            except KeyError:
                flat_dark_exptime = hdr["EXPOSURE"]

            try:
                flat_dark_xbin = hdr["XBINNING"]
            except:
                flat_dark_xbin = hdr["XBIN"]

            try:
                flat_dark_ybin = hdr["YBINNING"]
            except:
                flat_dark_ybin = hdr["YBIN"]

            logger.debug(f"Flat-dark frame readout mode: {flat_dark_readout_mode}")
            logger.debug(f"Flat-dark frame exposure time: {flat_dark_exptime}")
            logger.debug(f"Flat-dark frame X binning: {flat_dark_xbin}")
            logger.debug(f"Flat-dark frame Y binning: {flat_dark_ybin}")

    logger.info("Loading calibration frames...")
    if dark_frame is not None:
        logger.info(f"Loading dark frame: {dark_frame}")
        with fits.open(dark_frame) as hdul:
            dark = hdul[0].data
            hdr = hdul[0].header
        dark = dark.astype(np.float32)

        try:
            dark_readout_mode = hdr["READOUTM"]
        except KeyError:
            dark_readout_mode = hdr["READOUT"]

        try:
            dark_exptime = round(hdr["EXPTIME"], 3)
        except KeyError:
            dark_exptime = round(hdr["EXPOSURE"], 3)

        try:
            dark_xbin = hdr["XBINNING"]
        except:
            dark_xbin = hdr["XBIN"]

        try:
            ark_ybin = hdr["YBINNING"]
        except:
            dark_ybin = hdr["YBIN"]

        logger.debug(f"Dark frame readout mode: {dark_readout_mode}")
        logger.debug(f"Dark frame exposure time: {dark_exptime}")
        logger.debug(f"Dark frame X binning: {dark_xbin}")
        logger.debug(f"Dark frame Y binning: {dark_ybin}")

    if flat_frame is not None:
        logger.info(f"Loading flat frame: {flat_frame}")
        with fits.open(flat_frame) as hdul:
            flat = hdul[0].data
            hdr = hdul[0].header

        flat = flat.astype(np.float32)

        try:
            flat_filter = hdr["FILTER"]
        except KeyError:
            flat_filter = ""

        try:
            flat_readout_mode = hdr["READOUTM"]
        except KeyError:
            flat_readout_mode = hdr["READOUT"]

        try:
            flat_exptime = round(hdr["EXPTIME"], 3)
        except KeyError:
            flat_exptime = round(hdr["EXPOSURE"], 3)

        try:
            flat_xbin = hdr["XBINNING"]
        except:
            flat_xbin = hdr["XBIN"]

        try:
            flat_ybin = hdr["YBINNING"]
        except:
            flat_ybin = hdr["YBIN"]

        logger.debug(f"Flat frame readout mode: {flat_readout_mode}")
        logger.debug(f"Flat frame exposure time: {flat_exptime}")
        logger.debug(f"Flat frame X binning: {flat_xbin}")
        logger.debug(f"Flat frame Y binning: {flat_ybin}")

    logger.info("Looping through images...")
    for fname in fnames:
        with fits.open(fname) as hdul:
            image = hdul[0].data
            hdr = hdul[0].header

        image = image.astype(np.float32)

        try:
            image_filter = hdr["FILTER"]
        except KeyError:
            image_filter = ""

        try:
            image_readout_mode = hdr["READOUTM"]
        except KeyError:
            image_readout_mode = hdr["READOUT"]

        try:
            image_exptime = round(hdr["EXPTIME"], 3)
        except KeyError:
            image_exptime = round(hdr["EXPOSURE"], 3)

        try:
            image_xbin = hdr["XBINNING"]
        except:
            image_xbin = hdr["XBIN"]

        try:
            image_ybin = hdr["YBINNING"]
        except:
            image_ybin = hdr["YBIN"]

        logger.debug(f"Image readout mode: {image_readout_mode}")
        logger.debug(f"Image exposure time: {image_exptime}")
        logger.debug(f"Image X binning: {image_xbin}")
        logger.debug(f"Image Y binning: {image_ybin}")

        if image_readout_mode != dark_readout_mode:
            logger.warning(
                f"Image readout mode ({image_readout_mode}) does not match dark readout mode ({dark_readout_mode})"
            )

        if image_readout_mode != flat_readout_mode:
            logger.warning(
                f"Image readout mode ({image_readout_mode}) does not match flat readout mode ({flat_readout_mode})"
            )

        if image_xbin != dark_xbin:
            logger.warning(
                f"Image X binning ({image_xbin}) does not match dark X binning ({dark_xbin})"
            )

        if image_xbin != flat_xbin:
            logger.warning(
                f"Image X binning ({image_xbin}) does not match flat X binning ({flat_xbin})"
            )

        if image_ybin != dark_ybin:
            logger.warning(
                f"Image Y binning ({image_ybin}) does not match dark Y binning ({dark_ybin})"
            )

        if image_ybin != flat_ybin:
            logger.warning(
                f"Image Y binning ({image_ybin}) does not match flat Y binning ({flat_ybin})"
            )

        if image_filter != flat_filter:
            logger.warning(
                f"Image filter ({image_filter}) does not match flat filter ({flat_filter})"
            )

        if camera_type == "cmos":
            if image_exptime != dark_exptime:
                logger.warning(
                    f"""Image exposure time ({image_exptime}) does not match dark exposure time ({dark_exptime}),
                    recommended for a CMOS camera"""
                )
            if flat_exptime != flat_dark_exptime:
                logger.warning(
                    f"""Flat exposure time ({flat_exptime}) does not match flat-dark exposure time ({flat_dark_exptime}),
                    recommended for a CMOS camera"""
                )

        hdr.add_comment(f"Calibrated using pyscope")
        hdr.add_comment(f"Calibration mode: {camera_type}")
        hdr.add_comment(f"Calibration dark frame: {dark_frame}")
        hdr.add_comment(f"Calibration flat frame: {flat_frame}")
        hdr.add_comment(f"Calibration bias frame: {bias_frame}")
        hdr.add_comment(f"Calibration flat-dark frame: {flat_dark_frame}")
        hdr.add_comment(f"Calibration astro-scrappy: {astro_scrappy}")
        hdr.add_comment(f"Calibration bad columns: {bad_columns}")

        cal_image = image.copy()

        if camera_type == "ccd":
            if bias_frame is not None:
                logger.info("Applying bias frame (CCD selected)...")
                cal_image -= bias

            if dark_frame is not None:
                logger.info(
                    """Applying the dark frame. CCD selected so a bias-subtracted
                            dark frame is scaled by the ratio of the image exposure time over
                            the dark exposure time then subtracted from the image."""
                )

                cal_image -= (dark - bias) * (image_exptime / dark_exptime)
        elif camera_type == "cmos":
            if flat_dark_frame is not None:
                logger.info(
                    """Applying the flat-dark frame. CMOS selected so a dark
                            is subtracted from the image."""
                )
                cal_image -= dark

        if flat_frame is not None:
            logger.info("""Prepping the flat frame...""")

            if camera_type == "ccd":
                logger.debug(
                    """CCD selected so the flat frame is
                        bias-subtracted then a dark frame scaled in the same way as above
                        is subtracted from the flat frame, if those frames are present."""
                )
                if bias_frame is not None:
                    flat -= bias

                if dark_frame is not None:
                    flat -= (dark - bias) * (flat_exptime / dark_exptime)

            elif camera_type == "cmos":
                logger.debug(
                    """CMOS selected so the flat frame is simply
                        dark-flat subtracted if present."""
                )
                if flat_dark_frame is not None:
                    flat -= flat_dark

            logger.info(
                """Normalizing the flat frame by the mean of the
                        middle quarter of the image."""
            )
            x_size = flat.shape[1]
            y_size = flat.shape[0]
            x_start = int(x_size / 4)
            x_end = int(3 * x_size / 4)
            y_start = int(y_size / 4)
            y_end = int(3 * y_size / 4)

            flat_mean = np.mean(flat[y_start:y_end, x_start:x_end])
            flat /= flat_mean

            logger.debug("Clipping negative values to 1...")
            flat[flat <= 0] = 1

            logger.info("Applying the flat frame...")
            cal_image /= flat

        logger.info("Flooring the calibrated image...")
        cal_image = np.floor(cal_image)

        logger.info("Checking if pedestal is necessary...")
        if np.min(cal_image) < 0:
            logger.info(
                "Minimum value is %i, adding pedestal of equal value"
                % np.min(cal_image)
            )
            cal_image += np.abs(np.min(cal_image))
            hdr["PEDESTAL"] = np.abs(np.min(cal_image))

        if astro_scrappy[0] > 0:
            logger.info("Removing hot pixels...")
            t0 = time.time()
            mask, cal_image = astroscrappy.detect_cosmics(
                cal_image, niter=astro_scrappy[0], readnoise=astro_scrappy[1]
            )
            t = time.time() - t0
            hdr.add_comment(
                f"Removed hot pixels using astroscrappy, {astro_scrappy[0]} iterations"
            )
            hdr.add_comment("Hot pixel removal took %.1f seconds" % t)
            logger.debug(f"Hot pixel removal took {t} seconds")

        if len(bad_columns) > 0:
            logger.info("Fixing bad columns...")
            for badcol in bad_columns:
                cal_image[:, badcol] = (
                    cal_image[:, badcol - 1] + cal_image[:, badcol + 1]
                ) / 2

        logger.info("Clipping to uint16 range...")
        cal_image = np.clip(cal_image, 0, 65535)
        cal_image = cal_image.astype(np.uint16)

        logger.info("Writing calibrated status to header...")
        hdr["CALSTAT"] = True

        if in_place:
            logger.info(f"Overwriting {fname}")
            fits.writeto(fname, cal_image, hdr, overwrite=True)
        else:
            logger.info(f"Writing calibrated image to {fname}")
            fits.writeto(
                fname.split(".")[:-1] + "_cal.fts", cal_image, hdr, overwrite=True
            )

        logger.info("Done!")


ccd_calib = ccd_calib_cli.callback



#!/usr/bin/env python

# plot-photom: Plots Talon program photom output both vs. UT and phase
# RLM May 2016
# 26 May 2016 add JDrange option [-j]
# 27 Oct 2016 sort data by UT date, add period in hr to plot title
# 09 Dec 2016 RLM add error column on ASCII output file for Czech submissions
# 27 Mar 2017 check for reasonable V1, V1 err values, skip otherwise
# 1.41 reverse default y limits to 1,-1
# 1.50 Add reference line option, change output plot to PDF
# 1.60 Fixed O-C calculation, used numpy.polyfit for fitting, tweaked  phase plot, added width option for minimum fit
# 2.0 Switched to Gaussian fitting using scipy.curvefit [much better fits]
# 2.01 30 Oct 2018 add option to exclude output lines with calsource mag. much fainter than reference image magnitude
# 2.10 2 Nov 2018 switch to BJD time
# 2.11 23 Nov 2018 change default plot type to png
# 2.2 11-Dec-2019 make reading FITS files optional
# 3  Python 3 compatible, remove extraneous [?] imp library import

vers = "v.3.0  (4 March 2020)"

import math
import sys

# suppress warning message when object not found
import warnings
from operator import itemgetter
from optparse import OptionParser

import astropy.io.fits as pyfits
import ephem as ep  # pyephem library
import matplotlib.pyplot as plt
import numpy as np
from astropy import coordinates as coord
from astropy import time
from astropy import units as u
from astroquery.simbad import Simbad
from numpy import linalg
from scipy.optimize import curve_fit
from scipy.stats import chi2

warnings.filterwarnings("ignore")


def get_args():
    global parser
    d_txt = "Program plot-photom: plots output from program photom"
    parser = OptionParser(description=d_txt, version="%s" % vers)
    parser = OptionParser(
        description="%prog plots light curves using output file from program photom",
        version=vers,
    )
    parser.add_option(
        "-a",
        dest="plottype",
        metavar="plottype",
        action="store",
        default="png",
        help="Plot type [default png]",
    )
    parser.add_option(
        "-b",
        dest="barycenter",
        metavar="use barycenter time",
        action="store_true",
        default=False,
        help="Use barycenter time (requires FITS images) [def. False]",
    )
    parser.add_option(
        "-c",
        dest="check",
        metavar="show checkstar",
        action="store_true",
        default=False,
        help="Show check star False]",
    )
    parser.add_option(
        "-d",
        dest="double",
        metavar="show double",
        action="store_true",
        default=False,
        help="Show double phase (0.0-2.0) [default False]",
    )
    parser.add_option(
        "-P",
        dest="period",
        metavar="period",
        action="store",
        type=float,
        default=1,
        help="Period (days)",
    )
    parser.add_option(
        "-p",
        dest="plot_phase",
        metavar="plot_phase",
        action="store_true",
        default=False,
        help="Plot phase [default off]",
    )
    parser.add_option(
        "-j",
        dest="jdrange",
        metavar="jdmin, jdmax",
        action="store",
        default="0,0",
        help="JD range e.g. 2456722.73,2456724.56 [default all]",
    )
    parser.add_option(
        "-l",
        dest="line",
        metavar="line",
        action="store_true",
        default=False,
        help="Draw median line for check star",
    )
    parser.add_option(
        "-J",
        dest="JD0",
        metavar="JD0",
        action="store",
        type=float,
        default=5,
        help="Reference Barycentric Julian date, phase =0",
    )
    parser.add_option(
        "-m",
        dest="mag_ref",
        metavar="Ref. mag.",
        type=float,
        default=0.0,
        help="Reference star magnitude [default =0]",
    )
    parser.add_option(
        "-r",
        dest="refmag",
        metavar="refmag",
        action="store",
        type=float,
        default=1,
        help="Exclude images whose ref. star diff. mag exceeds refmag",
    )
    parser.add_option(
        "-t",
        dest="tmin",
        metavar="Time_minimum",
        action="store_true",
        default=False,
        help="Solve for time of minimum [default False]",
    )
    parser.add_option(
        "-w",
        dest="width",
        metavar="width",
        action="store",
        type=int,
        default=20,
        help="Minimum fit width, sample times [default 20]",
    )
    parser.add_option(
        "-T",
        dest="title",
        metavar="title",
        action="store",
        default="",
        help="Alternate plot suptitle [default object]",
    )
    parser.add_option(
        "-v",
        dest="verbose",
        metavar="verbose",
        action="store_true",
        default=False,
        help="Verbose output [default False]",
    )
    parser.add_option(
        "-y",
        dest="yminmax",
        metavar="ymin,ymax",
        action="store",
        default="1,-1",
        help="y axis min, max [default -1,1]",
    )

    return parser.parse_args()


def plot_lc_jd(
    bjd,
    ymin,
    ymax,
    jdmin,
    jdmax,
    obj,
    obj_sigma,
    ck,
    ck_sigma,
    pltname,
    plottype,
    suptitle,
    jd0,
    show_check,
    solve_tmin,
    tmin_args,
    box_text,
    line,
):
    fig, ax = plt.subplots(1)

    if solve_tmin:
        bjd_min, sigma, phs_min, jd1, oc, xmod, phs_mod, ymod = tmin_args
        bjd_min_frac = bjd_min - jd1
        title = (
            "BJD0: %.5f, P: %.9f, \n BJDmin: %.5f +/- %.5f, O-C: %.5f days [%.1f s+/- %.1f s]"
            % (jd0, P, bjd_min, sigma, oc, oc * 86400, sigma * 86400)
        )
    if jdmin != 0:
        jd_offset = jdmin
        xmax = jdmax - jdmin
    else:
        jd_offset = bjd[0]
        xmax = bjd[-1] - bjd[0]
    plt.xlim(0, xmax)
    plt.errorbar(
        bjd - jd_offset, obj, yerr=obj_sigma, ls="none", marker="o", markersize=3
    )
    if show_check:
        plt.errorbar(bjd - jd_offset, ck, yerr=ck_sigma, ls="none", marker=".")
    if use_barycenter:
        plt.xlabel("Barycentric JD - %.3f" % jd_offset)
    else:
        plt.xlabel("Heliocentric JD - %.3f" % jd_offset)
    if np.abs(ref_mag) < 0.01:
        plt.ylabel("Differential magnitude")
    else:
        plt.ylabel("Magnitude (Ref = %.2f)" % ref_mag)

    plt.grid(True)
    plt.suptitle(suptitle)
    if solve_tmin:
        plt.title(title, fontsize=9)
        plt.plot(xmod - jd_offset, ymod, "r-")
        plt.axvline(bjd_min - jd_offset, color="r", lw=1.5, linestyle="dashed")
    plt.ylim(ymin, ymax)
    if line:
        ymed = np.nanmedian(ck)
        plt.axhline(y=ymed, ls="dashed", color="green", label="Check star")
        plt.legend()

    # place a text box in lower right  in axes coords
    if use_barycenter:
        props = dict(boxstyle="round", facecolor="wheat", alpha=0.25)
        ax.text(
            0.75,
            0.15,
            box_text,
            transform=ax.transAxes,
            fontsize=8,
            verticalalignment="top",
            bbox=props,
        )

    plotname = pltname + "_lc_jd." + plottype
    plt.savefig(plotname)
    print("JD-magnitude plot file = %s" % plotname)
    return


def plot_lc_phs(
    phs,
    ymin,
    ymax,
    jdmin,
    jdmax,
    obj,
    obj_sigma,
    ck,
    ck_sigma,
    pltname,
    plottype,
    suptitle,
    jd0,
    show_check,
    show_double,
    solve_tmin,
    tmin_args,
    box_text,
):
    plt.figure()
    fig, ax = plt.subplots(1)
    if solve_tmin:
        bjd_min, sigma, phs_min, jd1, oc, xmod, phs_mod, ymod = tmin_args
        bjd_min_frac = bjd_min - jd1
        title = (
            "BJD0: %.5f, P: %.9f,\n BJDmin: %.5f +/- %.5f,  O-C: %.5f days [%.1f s +/- %.1f s]"
            % (jd0, P, bjd_min, sigma, oc, oc * 86400, sigma * 86400)
        )
    else:
        title = "BJD0: %.5f, P: %.9f day (%.3f hr)" % (jd0, P, P * 24.0)
    xmin = 0
    xmax = 1.0
    if show_double:
        xmax = 2.0
        phs = list(phs) + list(phs + 1)
        if solve_tmin:
            phs_mod = list(phs_mod) + list(phs_mod + 1)
            ymod = list(ymod) + list(ymod)
        obj = list(obj) + list(obj)
        obj_sigma = list(obj_sigma) + list(obj_sigma)
        ck = list(ck) + list(ck)
        ck_sigma = list(ck_sigma) + list(ck_sigma)

    plt.suptitle(suptitle)
    plt.title(title, fontsize=8)

    # place a text box in lower right  in axes coords
    props = dict(boxstyle="round", facecolor="wheat", alpha=0.25)
    ax.text(
        0.75,
        0.15,
        box_text,
        transform=ax.transAxes,
        fontsize=8,
        verticalalignment="top",
        bbox=props,
    )

    plt.errorbar(phs, obj, yerr=obj_sigma, ls="none", marker="o", markersize=2)
    if show_check:
        plt.errorbar(phs, ck, yerr=ck_sigma, color="g", ls="none", marker=".")
    if solve_tmin:
        plt.axvline(phs_min, color="r", lw=1.5, linestyle="dashed")
        plt.plot(phs_mod, ymod, "r.")
    plt.grid(True)
    plt.xlabel("Phase")
    plt.ylabel("Differential magnitude")
    plt.xlim(xmin, xmax)
    plt.ylim(ymin, ymax)

    plotname = pltname + "_lc_phase." + plottype
    plt.savefig(plotname)
    print("Phase-magnitude plot file = %s" % plotname)
    return


def phase_diff(jd, jd0, P):
    # Returns phase and o-c [day] at heliocentric observed date jd given ephemeris jd0,P
    jd_diff = jd - jd0
    phase = (jd_diff % P) / P
    if phase >= 0.5:
        o_c = (phase - 1) * P
    else:
        o_c = phase * P
    return phase, o_c


def fgauss(x, a, b, x0, w):
    t = (x - x0) ** 2 / w**2
    return a * np.exp(-t) + b


def calc_bjd(jd_utc, ra_str, dec_str):
    """
    Calculate barycentric dynamical Julian date from UTC Julian date, source coordinates
    Refs: http://docs.astropy.org/en/stable/time
    Eastman, et al. 2010 PASP,122,935
    """
    object = coord.SkyCoord(ra_str, dec_str, unit=(u.hourangle, u.deg), frame="icrs")
    lowell = coord.EarthLocation.of_site(
        "Multiple Mirror Telescope"
    )  # close enough to Winer
    times = time.Time(jd_utc, format="jd", scale="utc", location=lowell)
    ltt_bary = times.light_travel_time(object)
    bjd_tdb = times.tdb + ltt_bary
    return bjd_tdb.value


def get_hdr_info(fts_image):
    # returns usefule FITS header information, including Barycentric JD [calculated from RA,Dec,JD)
    try:
        hdr = pyfits.getheader(fts_image)
    except:
        sys.exit("Cannot find FITS image %s in current directory, exiting" % fts_image)
    ra_str = hdr["RA"]
    dec_str = hdr["DEC"]
    object = hdr["OBJECT"].replace(" ", "")
    filter = hdr["FILTER"]
    telescope = hdr["TELESCOP"]
    exptime = hdr["EXPTIME"]
    date_obs = hdr["DATE-OBS"][0:10].replace("-", "_")
    jd_utc = hdr["JD"] + exptime / (2.0 * 86400)
    bjd = calc_bjd(jd_utc, ra_str, dec_str)

    return object, ra_str, dec_str, exptime, filter, telescope, date_obs, jd_utc, bjd


# MAIN

deg = np.pi / 180.0

# Get command  line arguments, assign parameter values
(opts, args) = get_args()

fname = args[0]
plottype = opts.plottype
use_barycenter = opts.barycenter
show_check = opts.check
show_double = opts.double
refmag = opts.refmag
jd0 = opts.JD0
jdmin, jdmax = [float(x) for x in opts.jdrange.split(",")]
ref_mag = opts.mag_ref
line = opts.line
P = opts.period
plot_phase = opts.plot_phase
solve_tmin = opts.tmin
width = opts.width
suptitle = opts.title
ymin, ymax = [float(x) for x in opts.yminmax.split(",")]
verbose = opts.verbose

# Open photom output file
fn = open(fname, "r")
hdr = fn.readline()[1:-1]

# Read data
lines = fn.readlines()
BJD = []
ut_hr = []
obj = []
obj_sigma = []
ck = []
ck_sigma = []
phs = []
n = 0
for line in lines:
    mjd, dum, dum, a1, a2, a3, a4, a5, a6 = [float(x) for x in line.split()[6:]]
    if use_barycenter:
        ftsname = line.split()[1] + ".fts"
        if n == 0:
            (
                objname,
                ra_str,
                dec_str,
                exptime,
                filter,
                telescope,
                date,
                jd_utc,
                bjd,
            ) = get_hdr_info(ftsname)
        else:
            dum, dum, dum, dum, dum, dum, dum, jd_utc, bjd = get_hdr_info(ftsname)
    else:
        jd_utc = mjd + 2449000  # Heliocentric JD at start of exposure)
        bjd = jd_utc  # Hack!
        date = "JD_%7i" % int(jd_utc)  # why not
        objname = fname.split(".")[0]
        telescope = "Gemini"
        filter = ""
        exptime = 0
        ra_str = ""
        dec_str = ""

    if verbose:
        print(ftsname, jd_utc, bjd, a1)
    jd_ok = (jdmin == 0 and jdmax == 0) or jdmin <= bjd <= jdmax
    v1_ok = -8 < a1 < 8
    v1_sigma_ok = a2 < 1.0
    vref_ok = a5 < refmag
    if v1_ok and v1_sigma_ok and jd_ok and vref_ok:
        hr = math.modf(bjd + 0.5)[0] * 24
        ut_hr.append(hr)
        BJD.append(bjd)
        a1 += ref_mag
        a3 += ref_mag
        obj.append(a1)
        obj_sigma.append(a2)
        ck.append(a3)
        ck_sigma.append(a4)
        phs.append(((bjd - jd0) % P) / P)
        n += 1

if verbose:
    print("%i points read, computing plot..." % n)

# Time sort using BJD
vals = [
    [BJD[i], ut_hr[i], obj[i], obj_sigma[i], ck[i], ck_sigma[i], phs[i]]
    for i in range(len(BJD))
]
sorted_vals = sorted(vals, key=itemgetter(0))
for i in range(n):
    BJD[i], ut_hr[i], obj[i], obj_sigma[i], ck[i], ck_sigma[i], phs[i] = sorted_vals[i]

# Convert to numpy arrays, so means can be subtracted, and offsets applied
ut_hr = np.array(ut_hr)
BJD = np.array(BJD)
obj = np.array(obj)
obj_sigma = np.array(obj_sigma)
ck = np.array(ck)
ck_sigma = np.array(ck_sigma)

# If plotting differential magnitudes, subtract means
if np.abs(ref_mag < 0.01):
    obj -= np.mean(obj)
    ck -= np.mean(ck) + 0.0
phs = np.array(phs)

# Solve for time of minimum using weighted Gaussian LSQ fit centered on minimum of l.c.
if solve_tmin:
    jd_frac, jd_int = np.modf(BJD)
    jd1 = jd_int[0]  # Integer part of first BJD time
    jmin = np.argmax(obj)
    BJD_min0 = BJD[jmin]
    if verbose:
        print(
            "Found sample minimum at BJD = %.5f (jmin = %i, mag = %.2f)"
            % (BJD_min0, jmin, obj[jmin])
        )
    w = width / 2  # Width of fit in points
    x = BJD[jmin - w : jmin + w] - BJD[0]
    y = obj[jmin - w : jmin + w]
    s = obj_sigma[jmin - w : jmin + w]

    # weighted Gaussian fit
    init_vals = (obj[jmin], 0, BJD[jmin] - BJD[0], width * exptime / 86400)
    [a_fit, b_fit, x0_fit, w_fit], cov = curve_fit(fgauss, x, y, p0=init_vals)
    [a_s, b_s, x0_s, w_s] = np.sqrt(np.diag(cov))
    delta_jd = x0_fit
    sigma = x0_s  # Uncertainty in delta_jd
    bjd_min = BJD[0] + delta_jd
    phs_min, oc = phase_diff(bjd_min, jd0, P)

    # generate a model for plotting
    npts = 100
    xmod = np.linspace(BJD[jmin - w] - BJD[0], BJD[jmin + w] - BJD[0], npts)
    ymod = fgauss(xmod, a_fit, b_fit, x0_fit, w_fit)
    xmod += BJD[0]
    phs_mod = ((xmod - jd0) % P) / P
    if verbose:
        print(
            "Fitted minimum at BJD: %.5f +/- %.5f, O-C = %.5f days (%.1f sec +/- %.1f sec). Phase at minimum = %.3f)"
            % (bjd_min, sigma, oc, oc * 86400, sigma * 86400, phs_min)
        )
else:
    bjd_min = 0
    phs_min = 0
    oc = 0
    jd1 = 0
    sigma = 0
    xmod = 0
    phs_mod = 0
    ymod = 0

# Plot differential magnitude of target, check star vs heliocentric jd
pltname = "%s_%s" % (objname, date)
if suptitle == "":
    suptitle = objname
Ymin = np.mean(obj) + ymin
Ymax = np.mean(obj) + ymax
tmin_args = (bjd_min, sigma, phs_min, jd1, oc, xmod, phs_mod, ymod)
obj_args = (objname, ra_str, dec_str, exptime, filter, telescope, date)
box_text = "%s\nDate: %s\nFilter: %s\nExp time:  %.1f sec" % (
    telescope,
    date,
    filter,
    exptime,
)
plot_lc_jd(
    BJD,
    Ymin,
    Ymax,
    jdmin,
    jdmax,
    obj,
    obj_sigma,
    ck,
    ck_sigma,
    pltname,
    plottype,
    suptitle,
    jd0,
    show_check,
    solve_tmin,
    tmin_args,
    box_text,
    line,
)

# If requested, also plot vs. phase using user-supplied ephemeris
if plot_phase:
    plot_lc_phs(
        phs,
        ymin,
        ymax,
        jdmin,
        jdmax,
        obj,
        obj_sigma,
        ck,
        ck_sigma,
        pltname,
        plottype,
        suptitle,
        jd0,
        show_check,
        show_double,
        solve_tmin,
        tmin_args,
        box_text,
    )

# Write a 2-column bjd, magnitude output file(s)
outfile = pltname + "_jd.dat"
f = open(outfile, "w")
for j in range(len(BJD)):
    f.write("%.5f   %.3f   %.3f\n" % (BJD[j], obj[j], obj_sigma[j]))
f.close()
print("Wrote file %s" % outfile)

if plot_phase:
    outfile = pltname + "_phase.dat"
    f = open(outfile, "w")
    for j in range(len(ut_hr)):
        f.write("%.4f   %.3f   %0.3f\n" % (phs[j], obj[j], obj_sigma[j]))
    f.close()
    print("Wrote file %s" % outfile)



#!/usr/bin/env python
"""
align_images: Aligns FITS image[s] to a reference image.

INPUT: filepath1 and filepath2 pointing toward images
The first image is used as a reference, and the second image has its WCS information as well as size changed
to be pixel-aligned with the first image for easier comparison.
OUTPUT: Saves a copy of the aligned image as imagename_aligned.fits

Version 1.0 -- Developed by Jacob Isbell, 9 April 2017, modified for batch operation by RLM 16 April 2017
"""

vers = "%prog 1.0 16 Apr 2017"
import glob
import sys
from optparse import OptionParser
from sys import argv

import matplotlib.pyplot as plt
import numpy as np
from astropy.io import fits
from astropy.wcs import WCS


def get_args():
    usage = "Usage: %prog [options] comma-separated FITS files[s]"
    parser = OptionParser(description="Program %prog", usage=usage, version=vers)
    parser.add_option(
        "-r",
        dest="reference",
        metavar="Reference",
        action="store",
        help="Reference FITS image ",
    )
    parser.add_option(
        "-v",
        dest="verbose",
        metavar="Verbose",
        action="store_true",
        default=False,
        help="Verbose output",
    )

    return parser.parse_args()


def round_to_int(x):
    if x - int(x) >= 0.5:
        return int(x) + 1
    else:
        return int(x)


def align_and_crop(im1, im2):
    ref_image = fits.open(im1)[0]
    new_image = fits.open(im2)[0]

    # ---read the central ra and dec of the ref image ---
    ref_ra = ref_image.header["crval1"]
    ref_ctr_px1 = ref_image.header["crpix1"]
    ref_dec = ref_image.header["crval2"]
    ref_ctr_px2 = ref_image.header["crpix2"]
    ref_px_scale = abs(ref_image.header["amdx1"]) / degrees

    # ------ read in the same info for the other image ------
    new_ra = new_image.header["crval1"]
    new_ctr_px1 = new_image.header["crpix1"]
    new_dec = new_image.header["crval2"]
    new_ctr_px2 = new_image.header["crpix2"]
    new_px_scale = abs(new_image.header["amdy1"]) / degrees

    # ----- find the distance between the two image centers ------
    dist_dec = round_to_int((ref_dec - new_dec) / ref_px_scale)
    dist_ra = round_to_int((ref_ra - new_ra) / ref_px_scale)
    if verbose:
        print("Moving %i in x direction" % (dist_ra))
        print("Moving %i in y direction" % (dist_dec))

    if dist_ra > new_image.data.shape[0] or dist_dec > new_image.data.shape[1]:
        print(
            "The images are separated by too much distance and cannot be aligned. Exiting..."
        )
        sys.exit(1)

    new_im = []
    temp = np.copy(new_image.data)

    # sys.exit()
    if dist_dec >= 0:  # the new image moves up
        temp = np.roll(temp, dist_dec, axis=0)
        for x in range(temp.shape[0]):
            for y in range(dist_dec):
                temp[-y][x] = 0
    elif dist_dec < 0:  # the new image moves down
        temp = np.roll(temp, dist_dec, axis=0)
        for x in range(temp.shape[0]):
            for y in range(dist_dec):
                temp[y][x] = 0
    if dist_ra >= 0:  #  the new image moves left
        temp = np.roll(temp, dist_ra, axis=1)
        for y in range(temp.shape[1]):
            for x in range(dist_ra):
                temp[y][-x] = 0
    elif dist_ra < 0:  #  the new image moves right
        temp = np.roll(temp, dist_ra, axis=1)
        for y in range(temp.shape[1]):
            for x in range(dist_ra):
                temp[y][x] = 0

    """
	fig,(ax1,ax2,ax3) = plt.subplots(3)
	ax2.imshow(np.log10(temp),cmap='hot')
	ax1.imshow(np.log10(ref_image.data),cmap='hot')
	ax3.imshow(np.log10(temp-ref_image.data),vmin=-100,vmax=100,cmap='hot')
	plt.show()
	"""
    # ------ create the headers for the images based on their original headers

    new_hdu = fits.PrimaryHDU(temp)
    new_hdu.header = new_image.header
    new_hdu.header["crval1"] = ref_ra
    new_hdu.header["crval2"] = ref_dec
    # new_hdu.header['crpix1'] = new_image.data.shape[0]/2
    # new_hdu.header['crpix2'] = new_image.data.shape[1]/2
    # new_hdu.header['naxis1'] = new_image.data.shape[0]
    # new_hdu.header['naxis2'] = new_image.data.shape[1]

    return new_hdu


# ------------ MAIN --------------------

degrees = np.pi / 180.0

# Get command  line arguments, assign parameter values
(opts, args) = get_args()

ftsfiles = args[
    0
]  # FITS input file mask (either single file or wildcard - parsed by glob)
ref_image = opts.reference  # Reference FITS image name
verbose = opts.verbose  # Print diagnostics, more
for ftsfile in ftsfiles.split(","):
    ftsroot = ftsfile.split(".")[0]
    new_hdu = align_and_crop(ref_image, ftsfile)
    aligned_name = "%s_aligned.fts" % ftsroot
    new_hdu.writeto(aligned_name, overwrite=True)
    if verbose:
        print("Successfully aligned %s with %s" % (ftsfile, ref_image))
        print("Saved %s as %s" % (ftsfile, aligned_name))
        print()



from abc import ABC, abstractmethod

from ._docstring_inheritee import _DocstringInheritee


class Dome(ABC, metaclass=_DocstringInheritee):
    @abstractmethod
    def __init__(self, *args, **kwargs):
        pass

    @abstractmethod
    def AbortSlew(self):
        pass

    @abstractmethod
    def CloseShutter(self):
        pass

    @abstractmethod
    def FindHome(self):
        pass

    @abstractmethod
    def OpenShutter(self):
        pass

    @abstractmethod
    def Park(self):
        pass

    @abstractmethod
    def SetPark(self):
        pass

    @abstractmethod
    def SlewToAltitude(self, Altitude):
        pass

    @abstractmethod
    def SlewToAzimuth(self, Azimuth):
        pass

    @abstractmethod
    def SyncToAzimuth(self, Azimuth):
        pass

    @property
    @abstractmethod
    def Altitude(self):
        pass

    @property
    @abstractmethod
    def AtHome(self):
        pass

    @property
    @abstractmethod
    def AtPark(self):
        pass

    @property
    @abstractmethod
    def Azimuth(self):
        pass

    @property
    @abstractmethod
    def CanFindHome(self):
        pass

    @property
    @abstractmethod
    def CanPark(self):
        pass

    @property
    @abstractmethod
    def CanSetAltitude(self):
        pass

    @property
    @abstractmethod
    def CanSetAzimuth(self):
        pass

    @property
    @abstractmethod
    def CanSetPark(self):
        pass

    @property
    @abstractmethod
    def CanSetShutter(self):
        pass

    @property
    @abstractmethod
    def CanSlave(self):
        pass

    @property
    @abstractmethod
    def CanSyncAzimuth(self):
        pass

    @property
    @abstractmethod
    def ShutterStatus(self):
        pass

    @property
    @abstractmethod
    def Slaved(self):
        pass

    @property
    @abstractmethod
    def Slewing(self):
        pass



import pytest


def test_calibratoron(device, disconnect):
    device.CalibratorOn(1)
    assert device.Brightness == 1
    device.CalibratorOff()


def test_calibratoroff(device, disconnect):
    device.CalibratorOn(1)
    device.CalibratorOff()
    assert device.Brightness == 0


def test_opencover(device, disconnect):
    device.OpenCover()
    assert device.CoverState == 2


def test_closecover(device, disconnect):
    device.OpenCover()
    device.CloseCover()
    assert device.CoverState == 2


def test_haltcover(device, disconnect):
    device.OpenCover()
    device.HaltCover()
    assert device.CoverState == 4


def test_calibratorstate(device, disconnect):
    device.CalibratorOn(1)
    assert device.CalibratorState == 2


def test_maxbrightness(device, disconnect):
    assert device.MaxBrightness is not None
